import {Component, EventEmitter, Inject, Input, OnInit, Output} from '@angular/core';
import AuthApiService, {OAuthProviderData} from "@api/v1/authApi.service";
import {OAuthProvider} from "@model/oAuthProvider";
import {ToastrService} from "ngx-toastr";
import {
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogActions,
  MatDialogContent,
  MatDialogRef,
  MatDialogTitle
} from "@angular/material/dialog";
import {MatButton} from "@angular/material/button";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";
import {ActivatedRoute} from "@angular/router";
import {catchError, finalize, Observable, switchMap, tap} from "rxjs";
import {TwoFactorAuthentication} from "@model/twoFactorAuthentication";
import {OAuthAction} from "@model/oAuthAction";
import SetupService from "@api/v1/setup.service";

interface AvailableOAuthProviders extends OAuthProviderData {
  disabled: boolean;
  active: boolean;
}

@Component({
  selector: 'app-delete-confirm-dialog',
  template: `
    <h1 mat-dialog-title>Удалить провайдера?</h1>
    <mat-dialog-content>
      <p>Вы уверены, что хотите удалить провайдера {{ data.provider.name }}?</p>
    </mat-dialog-content>
    <mat-dialog-actions style="display: flex; justify-content: flex-end;">
      <button mat-button (click)="onCancel()">Отмена</button>
      <button mat-raised-button color="warn" (click)="onConfirm()">Удалить</button>
    </mat-dialog-actions>
  `,
  imports: [
    MatDialogTitle,
    MatDialogContent,
    MatDialogActions,
    MatButton
  ]
})
export class DeleteConfirmDialog {

  constructor(
    public dialogRef: MatDialogRef<DeleteConfirmDialog>,
    @Inject(MAT_DIALOG_DATA) public data: any
  ) {
  }

  onConfirm(): void {
    this.dialogRef.close(true);
  }

  onCancel(): void {
    this.dialogRef.close(false);
  }
}

@Component({
  selector: 'OAuthProviders',
  imports: [
    DataSpinnerComponent
  ],
  templateUrl: './OAuthProviders.html',
  styleUrl: './OAuthProviders.css',
  providers: [SetupService, AuthApiService]
})
export class OAuthProviders implements OnInit {
  protected providers: AvailableOAuthProviders[] = [];
  protected _activeProvidersId: OAuthProvider[] = [];
  protected _activeProviders: string[] = [];
  protected loading = true;

  @Input() message: string = 'Вы можете войти в аккаунт через';

  @Input() set activeProviders(data: string[]) {
    this._activeProviders = data;
    this.updateDisabledProviders();
  }

  @Input() set activeProvidersId(data: OAuthProvider[]) {
    this._activeProvidersId = data;
    this.updateDisabledProviders();
  }

  @Input() canUnlink: boolean = false;
  @Input() action: OAuthAction = OAuthAction.Login;
  @Input() isSetup: boolean = false;

  @Output() public oAuthUpdateProviders = new EventEmitter();
  @Output() public oAuthLoginResult: EventEmitter<TwoFactorAuthentication> = new EventEmitter();

  constructor(private setupApi: SetupService,
              private authApi: AuthApiService,
              private notify: ToastrService,
              private dialog: MatDialog,
              private route: ActivatedRoute) {
  }

  ngOnInit(): void {
    const fullUrl = `${window.location.protocol}//${window.location.host}${window.location.pathname}`;
    this.authApi.availableProviders(fullUrl).subscribe(providers => {
      this.updateDisabledProviders(providers);
    });

    this.route.queryParamMap
      .pipe(
        switchMap(params => {
          const result = params.get('result');

          if (!result) {
            this.loading = false; // Нет результата, завершение загрузки
            return [];
          }

          return this.handleOAuthResult(result); // Обрабатываем результат
        }),
        catchError(_ => {
          this.loading = false;
          return [];
        })
      )
      .subscribe();
  }

  private handleOAuthResult(result: string): Observable<any> {
    switch (this.action) {
      case OAuthAction.Login:
        return this.authApi.loginOAuth(result).pipe(
          tap(auth => {
            this.oAuthLoginResult.emit(auth);
          }),
          finalize(() => {
            this.loading = false;
          })
        );

      case OAuthAction.Bind:
        if (this.isSetup) {
          return this.setupApi.registerOAuth(result).pipe(
            tap(() => {
              this.oAuthUpdateProviders.emit();
            }),
            finalize(() => {
              this.loading = false;
            })
          );
        } else
          throw new Error('Action "Bind" requires setup mode to be enabled.');
        break;
      default:
        throw new Error('Unknown action type for action ' + this.action);
    }
  }

  private updateDisabledProviders(data: OAuthProviderData[] | null = null) {
    this.providers = (data ?? this.providers).map(provider => {
      return {
        ...provider,
        disabled: this._activeProvidersId.includes(provider.provider) || this._activeProviders.includes(provider.providerName),
        active: false
      };
    });
  }

  protected openOAuth(provider: AvailableOAuthProviders) {
    const oauthWindow = window.open(
      provider.redirect,
      '_self'
    );

    if (!oauthWindow) {
      this.notify.error('Не удалось открыть OAuth окно');
      return;
    }
  }

  protected confirmDelete(provider: AvailableOAuthProviders) {
    const dialogRef = this.dialog.open(DeleteConfirmDialog, {data: {provider}});

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.deleteProvider(provider);
      }
    });
  }

  protected deleteProvider(provider: AvailableOAuthProviders) {
    // todo: remove provider
  }
}
