import {Component} from '@angular/core';
import {MatSidenavContainer} from "@angular/material/sidenav";
import {MatFormFieldModule} from "@angular/material/form-field";
import {MatInput} from "@angular/material/input";
import {MatTooltip} from "@angular/material/tooltip";
import {MatButton} from "@angular/material/button";
import {MatCard} from "@angular/material/card";
import {FormBuilder, FormControl, FormGroup, ReactiveFormsModule, Validators} from "@angular/forms";
import {FocusNextDirective} from "@/directives/focus-next.directive";
import {DataSpinnerComponent} from "@component/common/data-spinner/data-spinner.component";
import AuthApiService from "@api/v1/authApi.service";
import {Router} from "@angular/router";
import {catchError} from "rxjs";
import {TwoFactorAuthentication} from "@model/twoFactorAuthentication";
import {PasswordInputComponent} from "@component/common/password-input/password-input.component";
import {OAuthProviders} from "@component/OAuthProviders/OAuthProviders";

@Component({
  selector: 'app-login',
  standalone: true,
  imports: [
    MatSidenavContainer,
    MatFormFieldModule,
    MatInput,
    MatTooltip,
    MatButton,
    MatCard,
    ReactiveFormsModule,
    FocusNextDirective,
    DataSpinnerComponent,
    PasswordInputComponent,
    OAuthProviders
  ],
  templateUrl: './login.component.html',
  styleUrl: './login.component.css',
  providers: [AuthApiService]
})
export class LoginComponent {
  protected loginForm!: FormGroup;
  protected loaderActive: boolean = false;
  protected loginButtonIsDisable: boolean = true;
  protected errorText: string = '';
  protected requiresTwoFactorAuth: boolean = false;

  constructor(private formBuilder: FormBuilder, private auth: AuthApiService, private router: Router) {
    this.auth.getRole()
      .subscribe(data => {
        if (data !== null)
          router.navigate(['admin']).then();
      });

    this.loginForm = this.formBuilder.group({
        user: ['',],
        password: ['',]
      }
    );

    this.loginForm.get('password')?.setValidators([
      Validators.required,
      Validators.minLength(8)
    ]);

    this.loginForm.get('user')?.setValidators([
      Validators.required,
      Validators.minLength(4)
    ]);

    this.loginForm.valueChanges.subscribe(() => {
      this.loginButtonIsDisable = !this.loginForm.valid;
    });
  }

  private updateTwoFactorValidation(data: TwoFactorAuthentication) {
    if (data === TwoFactorAuthentication.None) {
      this.router.navigate(['admin']).then();
      return;
    }

    this.requiresTwoFactorAuth = true;
    this.loginForm.addControl(
      'twoFactorCode',
      new FormControl('', Validators.required)
    );
    this.loginForm.removeControl('user');
    this.loginForm.removeControl('password');
    this.loginButtonIsDisable = !this.loginForm.valid;
  }

  protected login() {
    this.loaderActive = true;

    this.auth.login({
      username: this.loginForm.get('user')?.value,
      password: this.loginForm.get('password')?.value
    })
      .pipe(catchError(error => {
        this.loaderActive = false;
        this.errorText = error.error.detail;
        this.loginButtonIsDisable = true;
        throw error;
      }))
      .subscribe(x => {
        this.loaderActive = false;
        this.errorText = '';
        this.updateTwoFactorValidation(x);
      });
  }

  protected login2Fa() {
    this.loaderActive = true;

    this.auth.twoFactorAuth({
      code: this.loginForm.get('twoFactorCode')?.value,
      method: TwoFactorAuthentication.TotpRequired
    })
      .pipe(catchError(error => {
        this.loaderActive = false;
        this.errorText = error.error.detail;
        this.loginButtonIsDisable = true;
        throw error;
      }))
      .subscribe(_ => {
        this.loaderActive = false;
        this.errorText = '';
        this.router.navigate(['admin']).then();
      });
  }

  protected loginOAuth(result: TwoFactorAuthentication) {
    this.updateTwoFactorValidation(result);
  }
}
